/* 
 *  ================================================================
 *  CompositeHierarchy.java:
 *  ================================================================
 */ 

package composite02;

import java.lang.Math;
import java.util.List;
import java.util.ArrayList;
import java.awt.geom.*;   // Needed for affine transformation....
import java.awt.Graphics;
import java.awt.Graphics2D;
 
public class CompositeHierarchy implements Graphic {
    private boolean DEBUG = true;
    private static int iCurrentLevel = 0;
    private static double dCurrentXOffSet  = 0.0;
    private static double dCurrentYOffSet  = 0.0;
    private static double dCurrentRotation = 0.0;
    private static AffineTransform at = new AffineTransform();

    // Coordinates and orientation of composite graphic ...

    private double     xOffset;
    private double     yOffset;
    private double    rotation;

    // Collection of child graphics.

    private ArrayList<Graphic> children = new ArrayList<Graphic>();
 
    // Constructor for composite hierarchy ...

    public CompositeHierarchy( double xOffset, double yOffset, double rotation ) {
       this.xOffset  = xOffset;
       this.yOffset  = yOffset;
       this.rotation = rotation;
    }

    // Set Affine Fransformation  ...

    public void setAffineTransform ( AffineTransform at ) {
       this.at = at;
    }

    // Draw the graphic.

    public void draw( Graphics2D g2 ) {

        // Update parameters ...

        iCurrentLevel = iCurrentLevel + 1;
        dCurrentXOffSet  = dCurrentXOffSet  + xOffset;
        dCurrentYOffSet  = dCurrentYOffSet  + yOffset;
        dCurrentRotation = dCurrentRotation + rotation;
        at.translate( xOffset, yOffset );
        at.rotate( rotation );
        g2.setTransform( at );

        if ( DEBUG == true ) {
           System.out.printf("Start : Level= %2d\n", iCurrentLevel );
           System.out.printf("      : (x,y,rot) offset = (%6.1f,%6.1f,%6.2f)\n",
                                      dCurrentXOffSet, dCurrentYOffSet, dCurrentRotation );
           System.out.println( at );
        }

        for (Graphic graphic : children) {
             graphic.draw( g2 );
        }

        if ( DEBUG == true ) {
           System.out.printf("End Level: %2d\n", iCurrentLevel );
        }

        // Decrement level, x- and y- offsets and rotation....

        iCurrentLevel    = iCurrentLevel - 1;
        dCurrentXOffSet  = dCurrentXOffSet  - xOffset;
        dCurrentYOffSet  = dCurrentYOffSet  - yOffset;
        dCurrentRotation = dCurrentRotation - rotation;
        at.rotate( -rotation );
        at.translate( -xOffset, -yOffset );
        g2.setTransform( at );
    }
 
    // Adds the graphic to the composition.

    public void add(Graphic graphic) {
        children.add(graphic);
    }
 
    //Removes the graphic from the composition.

    public void remove(Graphic graphic) {
        children.remove(graphic);
    }
}

