/**
  *  =====================================================================
  *  SimplePolygon.java: A simple polygon model ...
  *  =====================================================================
  */

package visitor02.model.primitive;

import java.lang.Math;
import java.util.ArrayList;

import visitor02.model.*;

public class SimplePolygon extends AbstractFeature {
   double dMinX, dMaxX, dMinY, dMaxY;
   double [] x;
   double [] y;

   public SimplePolygon( double xPoints[], double yPoints[] ) {
      super();

      // Make a copy of the incoming array coordinates ...

      x = (double[])xPoints.clone();
      y = (double[])yPoints.clone();

      // Compute max and min polygon coordinates ...

      dMinX = dMaxX = x[0];
      dMinY = dMaxY = y[0];

      for ( int k = 1; k < x.length; k++ ) {
         dMinX = Math.min ( dMinX, x[k] ); 
         dMaxX = Math.max ( dMaxX, x[k] ); 
         dMinY = Math.min ( dMinY, y[k] ); 
         dMaxY = Math.max ( dMaxY, y[k] ); 
      }

      // Save anchor reference point for shape ...

      setX ( dMinX );
      setY ( dMinY );
      setWidth(  dMaxX - dMinX );
      setHeight( dMaxY - dMinY );
      be.addPoint ( dMinX, dMinY );
      be.addPoint ( dMaxX, dMaxY );
   }

   // Clone (i.e., deep copy) simple polygon object ...

   public SimplePolygon clone() {
       try {
           SimplePolygon cloned = (SimplePolygon) super.clone();
           return cloned;
       } catch ( CloneNotSupportedException e) {
           System.out.println("*** Error in SimplePolygon.clone() ... ");
           return null;
       }
   }

   // Create string representation of a point ...
    
   public String toString() {
      String s = "SimplePolygon(" + getName() + ")\n";
      s = s + "-- Anchor (x,y) = (" + getX() + ", " + getY() + ")\n";
      s = s + "-- Width " + getWidth() + " height = " + getHeight();
      return s;
   }
}
