/*
 * ===================================================================
 * AbstractFeature.java: Abstract class for geographic features ....
 * ===================================================================
 */

package visitor03.model;

import java.awt.geom.*;  // Needed for affine transformation....

public abstract class AbstractFeature implements Feature {
   protected Location            c;
   protected BoundingEnvelope   be;
   protected String sName;
   protected double height;
   protected double width;
   protected boolean DEBUG = true;

   public AbstractFeature () {
     c  = new Location();
     be = new BoundingEnvelope();
   }

   public AbstractFeature ( double dX, double dY ) {
     c  = new Location( dX, dY );
     be = new BoundingEnvelope( dX, dY );
   }

   public void setName( String sName ) {
     this.sName = sName;
   }

   public String getName() {
     return sName;
   }

   // Methods to set/get the feature location ...

   public void setX( double dX ) {
      c.setX(dX);
   }

   public double getX() { 
      return c.getX();
   }

   public void setY( double dY ) {
      c.setY(dY);
   }

   public double getY() { 
      return c.getY();
   }

   // ================================================
   // New position and/or movement ...
   // ================================================

   public void setPosition( double dX, double dY ) {
      if ( DEBUG == true ) {
         System.out.printf("*** In AbstractFeature.setPosition(): Setting c.dX = %f ... \n", dX );
         System.out.printf("*** In AbstractFeature.setPosition(): Setting c.dY = %f ... \n", dY );
      }
      be.setToNull();
      c.setPosition(dX,dY);
   }

   // Methods to set/get the feature height and width ...

   public void setHeight( double height ) {
     this.height = height;
   }

   public double getHeight() { 
     return height;
   }

   public void setWidth( double width ) {
     this.width = width;
   }

   public double getWidth() { 
     return width;
   }

   // Accept method for visiting descendants of AbstractFeature ...
 
   public void accept( FeatureElementVisitor visitor ) {     
      visitor.visit( this );
   }

   // Clone (i.e., deep copy) abstract feature contents ...

   public AbstractFeature clone() throws CloneNotSupportedException {
      try {
          AbstractFeature cloned = (AbstractFeature) super.clone();
          cloned.c  = (Location) this.c.clone();
          cloned.be = (BoundingEnvelope) this.be.clone();
          return cloned;
      } catch ( CloneNotSupportedException e) {
          System.out.println("*** Error in AbstractFeature.clone() ... ");
          return null;
      }
   }
}

