/*
 *  ==============================================================
 *  Location.java: Store (x,y,z) location of a geographic feature.
 *  
 *  Written by: Mark Austin                             March 2016  
 *  ==============================================================
 */

package visitor03.model;

import java.util.*;

public class Location implements Cloneable {
   private List<LocationListener> listeners = new ArrayList<LocationListener>();
   private double dX, dY;

   // Constructor methods ....

   public Location() { } 

   public Location( double dX, double dY ) {
      this.dX = dX;
      this.dY = dY;
   } 

   // Methods to set and get nodal coordinates ..

   public void setX( double dX ) {
      this.dX = dX;
   }

   public void setY( double dY ) {
      this.dY = dY;
   }

   public double getX() { return dX; }
   public double getY() { return dY; }

   // Support for location listeners ...

   public void addLocationListener( LocationListener listener) {
      listeners.add(listener);
   }

   public void removeLocationListener( LocationListener listener) {
      listeners.remove(listener);
   }

   // New position (dX, dY) ...

   public void setPosition( double dX, double dY ) {
      if ( this.dX != dX || this.dY != dY ) {
         System.out.println("New location: ... ");
         this.dX = dX; this.dY = dY;
         notifyListeners();
      }
   }

   // Move a distance (dx, dy) ....

   public void move( double dx, double dy ) {
      if ( dx != 0.0 || dy != 0.0 ) {
         System.out.println("Location moved: ... ");
         dX = dX + dx;
         dY = dY + dy;
         notifyListeners();
      }
   }

   // Notify all registered listeners.

   private void notifyListeners() {
      LocationEvent evt = new LocationEvent(this);
      for (LocationListener aListener : listeners) {
           aListener.nodeMoved(evt);
      }
   }

   // Create clone (i.e., deep copy) of Location object ...

   public Object clone() {
      try {
         Location coord = (Location) super.clone();
         return coord; 
      } catch (CloneNotSupportedException e) {
         return null;
      }
   }
}
