/**
  *  =====================================================================
  *  HouseModel.java: Construct a simple 2D model for a House.
  *
  *     @param x the left of the bounding rectangle
  *     @param y the top of the bounding rectangle
  *     @param width the width of the bounding rectangle
  *  =====================================================================
  */

package visitor03.model.compound;

import visitor03.model.*;
import visitor03.model.primitive.*;

public class HouseModel extends AbstractCompoundFeature implements LocationListener {
   public boolean DEBUG = true;
   public Point node01, node02, node03, node04, node05;
   public Edge  edge01, edge02, edge03, edge04, edge05, edge06;

   public HouseModel( double x, double y, double width) {

      // Create label for the house model ...

      setName( "House Model" );

      // Save corner reference point for shape ...

      setX( (double) x );
      setY( (double) y );
      setHeight( (double) width );
      setWidth(  (double) width );

      // Initialize envelope ...
    
      be.addPoint( x, y );
      be.addPoint( x + width, y + width );

      // Create vertices for house schematic ...

      node01 = new Point(    "base01",             x,             y );
      node02 = new Point(    "base02",     x + width,             y );
      node03 = new Point( "ceiling01",             x, y + width/2.0 );
      node04 = new Point( "ceiling02",     x + width, y + width/2.0 );
      node05 = new Point(    "roof01", x + width/2.0,     y + width );

      // Connect edges to vertices ....

      edge01 = new Edge( "edge01",  node01, node02 );
      edge02 = new Edge( "edge02",  node01, node03 );
      edge03 = new Edge( "edge03",  node02, node04 );
      edge04 = new Edge( "edge04",  node03, node04 );
      edge05 = new Edge( "edge05",  node03, node05 );
      edge06 = new Edge( "edge06",  node04, node05 );

      // Add points to the model ...

      items.put( node01.getName(), node01 );
      items.put( node02.getName(), node02 );
      items.put( node03.getName(), node03 );
      items.put( node04.getName(), node04 );
      items.put( node05.getName(), node05 );

      // Add edges to the model ...

      items.put( edge01.getName(), edge01 );
      items.put( edge02.getName(), edge02 );
      items.put( edge03.getName(), edge03 );
      items.put( edge04.getName(), edge04 );
      items.put( edge05.getName(), edge05 );
      items.put( edge06.getName(), edge06 );

      // Add this item to list of location listeners ....

      c.addLocationListener ( this );
   }

   // Implementation of location event handlers

   public void nodeMoved( LocationEvent evt ) {
      if ( DEBUG == true ) {
         System.out.println("*** In model.compound.HouseModel.nodeMoved() ... ");
      }

      // Retrieve new position ....

      double x = getX();
      double y = getY();

      // Update nodal coordinates ...

      node01.setX ( x );             node01.setY ( y );
      node02.setX ( x + width );     node02.setY ( y );
      node03.setX ( x );             node03.setY ( y + width/2.0 );
      node04.setX ( x + width );     node04.setY ( y + width/2.0 );
      node05.setX ( x + width/2.0 ); node05.setY ( y + width );

      // Update (x,y) coordinates for the edges ...

      edge01.update( node01, node02 );
      edge02.update( node01, node03 );
      edge03.update( node02, node04 );
      edge04.update( node03, node04 );
      edge05.update( node03, node05 );
      edge06.update( node04, node05 );

      // Update bounding envelope ...

      be.setToNull();
      be.addPoint( x, y );
      be.addPoint( x + width, y + width );
   }

   // Accept method for Feature Element visitor ...

   public void accept( FeatureElementVisitor visitor) {
        visitor.visit(this);
   }

   // ==============================================================
   // Create string representation of car model ....
   // ==============================================================

   public String toString() {
      String s = "House Model: Name = " + sName + "\n";
      s = s    + "     " + "Location (x,y) = (" + getX() + "," + getY() + ")";

      return s;
   }
}
